/*************************************************************************
 *
 *   Used with ICCARM and AARM.
 *
 *    (c) Copyright IAR Systems 2009
 *
 *    File name   : suart.c
 *    Description : software uart driver for LPC29xx
 *
 *    History :
 *    1. Date        : 24.11.2009 
 *       Author      : Stoyan Choynev
 *       Description : Initial revision
 *
 *    $Revision: 1.0 $
 **************************************************************************/

/** include files **/
#include "LPC29xx.h"  /* LPC29xx definitions */
#include "type.h"
#include "irq.h"
#include <intrinsics.h>
#include "suart.h"        /**/
#include "suart_cfg.h"    /**/

/*Timer definitions*/
#define TCR             (0x000)
#define TC              (0x004)
#define PR              (0x008)
#define MCR             (0x00C)
#define EMR             (0x010)
#define MR0             (0x014)
#define MR1             (0x018)
#define MR2             (0x01C)
#define MR3             (0x020)
#define CCR             (0x024)
#define CR0             (0x028)
#define CR1             (0x02C)
#define CR2             (0x030)
#define CR3             (0x034)
#define INT_CLR_ENABLE  (0xFD8)
#define INT_SET_ENABLE  (0xFDC)
#define INT_STATUS      (0xFE0)
#define INT_ENABLE      (0xFE4)
#define INT_CLR_STATUS  (0xFE8)
#define INT_SET_STATUS  (0xFEC)

#define TIMER_SET_REG(base,reg, value)        (*(volatile unsigned int *)(base + reg)) = (value)

#define SFSP_GPIO_NO_PUPD 0x04
#define SFSP_GPIO_PU      0x0C
#define SFSP_GPIO_PD      0x14
#define SFSP_GPIO_BK      0x1C

#define MAKE_PORT_INPUT(port,mask)  (*(volatile unsigned int *)(port + 0x008)) &= ~(mask)
#define MAKE_PORT_OUTPUT(port,mask) (*(volatile unsigned int *)(port + 0x008)) |=  (mask)
#define SET_PORT(port,mask)         (*(volatile unsigned int *)(port + 0x004)) |=  (mask)
#define CLEAR_PORT(port,mask)       (*(volatile unsigned int *)(port + 0x004)) &= ~(mask)
#define GET_PORT(port,mask)         (*(volatile unsigned int *)(port + 0x000)) & (mask)



typedef struct
{
  char Data;
  char Status;
}t_rxdata;                        /*data array*/

typedef struct
{
  t_rxdata Buffer[RX_BUFF_SIZE];  /*data array*/
  t_rxdata * pWrite;              /*Write pointer*/
  t_rxdata * pRead;               /*Read pointer*/
  char flBuffNE;                  /*Not Empty Flag*/
} t_RXBUFF;

typedef struct
{
  char Buffer[TX_BUFF_SIZE];      /*data array*/
  char * pWrite;                  /*Write pointer*/
  char * pRead;                   /*Read pointer*/
  char flBuffF;                   /*Buffer full Flag*/
} t_TXBUFF;
/** default settings **/

/** external functions **/

/** external data **/

/** internal functions **/
static void SUART_RX_Handler(void);
static void SUART_TX_Handler(void);

/** public data **/

/** private data **/
static t_RXBUFF RxBuffer;
static t_TXBUFF TxBuffer;
static int RxScanCntr;
static int TxScanCntr;
/** public functions **/

/*************************************************************************
 * Function Name: SUART_Init
 * Parameters: None
 *
 * Return: None
 *
 * Description:SUART Initialization. Installs RX Timer interrupt and
 *             enables it.
 *             Clears RX and TX Buffers and init some internal variables.
 *
 *************************************************************************/
void SUART_Init(void)
{
  /*Configure RX Pin*/
  SURX_SFSP = SFSP_GPIO_NO_PUPD;
  MAKE_PORT_INPUT(SURX_PORT,SURX_MASK);
  /*Configure TX Pin*/
  SET_PORT(SUTX_PORT,SUTX_MASK);
  SUTX_SFSP = SFSP_GPIO_NO_PUPD;
  MAKE_PORT_OUTPUT(SUTX_PORT,SUTX_MASK);
  /*RX timer initilaization*/
  TIMER_SET_REG(RX_TIMER,TCR,2);  /*Stop And reset Timer*/
  TIMER_SET_REG(RX_TIMER,PR,0);   /*No prescaler*/
  TIMER_SET_REG(RX_TIMER,MCR,1);  /*Reset On mach 0*/
  TIMER_SET_REG(RX_TIMER,MR0,((RX_TIMER_CLK+SUART_BAUD_RATE/2)/SUART_BAUD_RATE+5/2)/5);     /*5 time faseter scan*/
  TIMER_SET_REG(RX_TIMER,INT_CLR_STATUS,0xFFFFFFFF);  /*Clear all interrupts*/
  TIMER_SET_REG(RX_TIMER,INT_SET_ENABLE,1);  /*Enable interrupt on mach 0*/
  /*TX timer initilaization*/
  TIMER_SET_REG(TX_TIMER,TCR,2);  /*Stop And reset Timer*/
  TIMER_SET_REG(TX_TIMER,PR,0);   /*No prescaler*/
  TIMER_SET_REG(TX_TIMER,MCR,1);  /*Reset On mach 0*/
  TIMER_SET_REG(TX_TIMER,MR0,(RX_TIMER_CLK+SUART_BAUD_RATE/2)/SUART_BAUD_RATE);     /*5 time faseter scan*/
  TIMER_SET_REG(TX_TIMER,INT_CLR_STATUS,0xFFFFFFFF);  /*Clear all interrupts*/
  TIMER_SET_REG(TX_TIMER,INT_SET_ENABLE,1);  /*Enable interrupt on mach 0*/
  /*Instal interrupt*/
  install_irq( SUART_RX_INT, (void *)SUART_RX_Handler, HIGHEST_PRIORITY );
  EnableIntReq_IRQ( SUART_RX_INT, ACTIVE_HIGH, HIGHEST_PRIORITY );
  /*Instal interrupt*/
  install_irq( SUART_TX_INT, (void *)SUART_TX_Handler, HIGHEST_PRIORITY );
  EnableIntReq_IRQ( SUART_TX_INT, ACTIVE_HIGH, HIGHEST_PRIORITY );
  /**/
  RxScanCntr = 0;
  TxScanCntr = 0;
  TxBuffer.pRead = TxBuffer.pWrite = TxBuffer.Buffer;
  RxBuffer.pRead = RxBuffer.pWrite = RxBuffer.Buffer;
  (RxBuffer.pRead)->Status = 0;
  TIMER_SET_REG(RX_TIMER,TCR,1);  /*Start Timer*/
}

/*************************************************************************
 * Function Name: SUART_GetStatus
 * Parameters: None
 *
 * Return:char - SUART Status
 *
 * Description: Returns SUART Status. bit 0 - Framming error
 *                                    bit 1 - Rx Buffer Not Empty
 *                                    bit 2 - Tx Buffer Full
 *************************************************************************/
char SUART_GetStatus(void)
{
  return TxBuffer.flBuffF | RxBuffer.flBuffNE | (RxBuffer.pRead)->Status;
}

/*************************************************************************
 * Function Name: SUART_GeChar
 * Parameters:None
 *
 * Return:char Data
 *
 * Description: Reads one byte from Rx Buffer
 *
 *************************************************************************/
char SUART_GetChar(void)
{
  char Data = (RxBuffer.pRead)->Data;
  
  if(RxBuffer.flBuffNE)
  {
    if((RxBuffer.Buffer + RX_BUFF_SIZE) == ++RxBuffer.pRead)
    {
      RxBuffer.pRead = RxBuffer.Buffer;
    }
    
    __disable_interrupt();
    
    if(RxBuffer.pWrite == RxBuffer.pRead)
    {
      RxBuffer.flBuffNE = 0;
    }
    __enable_interrupt();
  }
  
  return Data;
}
/*************************************************************************
 * Function Name: SUART_GeChar
 * Parameters:Data
 *
 * Return:char None
 *
 * Description: Pits one byte to Tx Buffer
 *
 *************************************************************************/
void SUART_PutChar(char data)
{
  
  if(!TxBuffer.flBuffF)
  {/*if buffer is not full*/
    *(TxBuffer.pWrite) = data; /*Put data in buffer*/
    /*Increment prointer*/
    if((TxBuffer.Buffer + TX_BUFF_SIZE) == ++TxBuffer.pWrite)
    {
      TxBuffer.pWrite = TxBuffer.Buffer;
    }
    
    __disable_interrupt();
    /*Check buffer full*/
    if(TxBuffer.pWrite == TxBuffer.pRead)
    {/*If buffer is full set flag*/
      TxBuffer.flBuffF = TXB_FULL;
    }
    __enable_interrupt();
    
    TIMER_SET_REG(TX_TIMER,TCR,1);  /*Start Timer*/   
  }
}
/** private functions **/
static void SUART_RX_Handler(void)
{
  TIMER_SET_REG(RX_TIMER,INT_CLR_STATUS,1);  /*Clear interrupt*/
  if( 2 <= RxScanCntr)
  {/*Receive*/
    if(10>RxScanCntr)
    {/*Data receive*/
      RxScanCntr++;

      (RxBuffer.pWrite)->Data >>= 1;
      if(GET_PORT(SURX_PORT,SURX_MASK))
      {
        (RxBuffer.pWrite)->Data |= 0x80;
      }
      else
      {
        (RxBuffer.pWrite)->Data &= ~0x80;
      }
    }
    else
    {/*Stop bit check*/
      if(10 == RxScanCntr)
      {/*Scan fasetr*/
        TIMER_SET_REG(RX_TIMER,MR0,((RX_TIMER_CLK+SUART_BAUD_RATE/2)/SUART_BAUD_RATE+5/2)/5);     /*5 time faseter scan*/
        RxScanCntr++;
      }
      
      if(GET_PORT(SURX_PORT,SURX_MASK))
      {/*Stop bit found. Resynch*/
        if((RxBuffer.Buffer + RX_BUFF_SIZE) == ++RxBuffer.pWrite)
        {
          RxBuffer.pWrite = RxBuffer.Buffer;
        }
        
        RxBuffer.flBuffNE = RXB_NE; 
        RxScanCntr = 0;
      }
      else
      {/*Stop bit error*/
        (RxBuffer.pWrite)->Status = FRM_ERR;
      }
    }
  }
  else
  {/*Wait start bit*/
    if(GET_PORT(SURX_PORT,SURX_MASK))
    {/*Rx pin is in high level*/
      RxScanCntr = 0;
    }
    else
    {/*Rx pin is in low level*/
      if(2 == ++RxScanCntr)
      {/*Valid Start bit found. Start receiving*/
       (RxBuffer.pWrite)->Status = 0;
       
        TIMER_SET_REG(RX_TIMER,MR0,(RX_TIMER_CLK+SUART_BAUD_RATE/2)/SUART_BAUD_RATE);     /*Scan at Baud rate*/
      }
    }
  }
}

static void SUART_TX_Handler(void)
{
  if(0 == TxScanCntr)
  {/*Start Bit*/
    CLEAR_PORT(SUTX_PORT,SUTX_MASK); 
    TxScanCntr++;
  }
  else
  {/**/
    if(9 == TxScanCntr)
    {/*Stop bit*/
     SET_PORT(SUTX_PORT,SUTX_MASK); 
     /**/
     TxScanCntr = 0;
      /*Inclrement pointer*/
      if((TxBuffer.Buffer + TX_BUFF_SIZE) == ++TxBuffer.pRead)
      {
        TxBuffer.pRead = TxBuffer.Buffer;
      }
      /*Clear buffer full flag*/
      TxBuffer.flBuffF = 0;
            
      if(TxBuffer.pWrite == TxBuffer.pRead)
      {/*If no more data in the buffer*/
        TIMER_SET_REG(TX_TIMER,TCR,2);  /*Stop And reset Timer*/
      }
    }
    else
    {/*Data transmit*/
      if( *(TxBuffer.pRead) & 0x1)
      {/*Send one*/
         SET_PORT(SUTX_PORT,SUTX_MASK); 
      }
      else
      {/*Send zero*/
        CLEAR_PORT(SUTX_PORT,SUTX_MASK); 
      }
      /*Shift data*/
      *(TxBuffer.pRead) >>= 1;
      TxScanCntr++;
    }
  }
  
  TIMER_SET_REG(TX_TIMER,INT_CLR_STATUS,1);  /*Clear interrupt*/
}